#!/usr/bin/env python3

import sys
import os
import string
import numpy as np
import h5py
import matplotlib
for backend in ['Qt5Agg', 'TkAgg']:
    try:
        matplotlib.use(backend)
        import matplotlib.pyplot as plt
        plt.close(plt.figure())
        print(f"Using Matplotlib backend: {backend}")
        break
    except ImportError:
        pass
else:
    import matplotlib.pyplot as plt

def print_usage():
    print(f"Usage: '{os.path.basename(__file__)} <ngauge output file> <counter ID> [<counter ID> ...]'", file=sys.stderr)


def fail():
    print_usage()
    sys.exit(1)


def list_datasets(file):
    with h5py.File(file,'r') as f:
        print(f"The following counters are available in file {os.path.basename(file)}:", file=sys.stderr)
        for k in f.keys():
            print(hex(int(k)))


def display(file, counters):
    fig = plt.figure()
    units = set()
    normalized = True
    with h5py.File(file,'r') as f:
        try:
            fw_ver = f.attrs["firmware_version"]
            dev_type = f.attrs["device_type"]
            fig.suptitle(f"Device: {dev_type}; Firmware: {fw_ver}")
        except KeyError:
            pass
        for c in counters:
            try:
                dset = f[c]
            except KeyError:
                list_datasets(sys.argv[1])
                sys.exit(1)
            unit = dset.attrs["unit"]
            units.add(unit)

        units = list(units)
        gs = fig.add_gridspec(len(units), hspace=0)
        axs = gs.subplots(sharex=True)
        for c in counters:
            dset = f[c]
            unit = dset.attrs["unit"]
            name = dset.attrs["name"]
            try:
                normalizer = dset.attrs["normalizer"]
            except KeyError:
                if normalized:
                    print("WARNING: This data is from an older version of ngauge. Plotting un-normalized.", file=sys.stderr)
                    normalized = False
                normalizer = "N/A"
            raw_start_times = dset[1:, 0]
            start_time = (raw_start_times - dset[1, 0]) * 1e-9
            end_time = (dset[1:, 1] - dset[1, 0]) * 1e-9
            time = (start_time + end_time) / 2.0
            time_error = (end_time - start_time) / 2.0
            common_current_indexes = np.arange(len(time))
            if normalizer == "N/A":
                normalizer = 1
            elif normalizer == "time":
                normalizer = (np.diff(dset[:, 0] - dset[0, 0])) * 1e-9
            elif normalizer[:3] == "id/":
                normalizer_id = str(int(normalizer.split('/', maxsplit=1)[1], 16))
                normalizer_raw_start_times = f[normalizer_id][1:, 0]
                _, common_current_indexes, common_normalizer_indexes = \
                    np.intersect1d(raw_start_times, normalizer_raw_start_times,
                                   assume_unique=True, return_indices=True)
                time = time[common_current_indexes]
                normalizer = f[normalizer_id][1:, 2][common_normalizer_indexes]
                normalizer = normalizer.astype(np.float64)
                normalizer[normalizer==0] = np.nan

            data = dset[1:, 2][common_current_indexes] / normalizer
            normalizer_error = time_error / normalizer
            data_error = dset[1:, 2][common_current_indexes] * normalizer_error / normalizer

            try:
                axs[units.index(unit)].errorbar(time, data, xerr=time_error, marker='x', label=f"{name} ({hex(int(c))})", linestyle='none')
                axs[units.index(unit)].fill_between(time, data - data_error, data + data_error, alpha=0.2)
                axs[units.index(unit)].legend()
            except TypeError:
                axs.errorbar(time, data, xerr=time_error, marker='x', label=f"{name} ({hex(int(c))})", linestyle='none')
                axs.fill_between(time, data - data_error, data + data_error, alpha=0.2)
                axs.legend()

    #axs[0].set_yscale('log', base=2)  # Uncomment per plot, if needed
    fig.supylabel('Counts')
    plt.xlabel('Time (s)')
    plt.show()


if __name__ == '__main__':
    if len(sys.argv) < 3:
        fail()

    if not os.path.isfile(sys.argv[1]):
        fail()

    counters = []
    for cnt in sys.argv[2:]:
        if cnt.isdigit():
            counters.append(cnt)
        elif cnt.startswith("0x") and all(c in string.hexdigits for c in cnt[2:]):
            counters.append(str(int(cnt, 16)))
        else:
            fail()

    display(sys.argv[1], counters)
