#!/usr/bin/env python3

import sys
import os
import string
import numpy as np
import h5py
import plotext as plt


def print_usage():
    print(f"Usage: '{os.path.basename(__file__)} <ngauge output file> <counter ID> [<counter ID> ...]'", file=sys.stderr)


def fail():
    print_usage()
    sys.exit(1)


def list_datasets(file):
    with h5py.File(file,'r') as f:
        print(f"The following counters are available in file {os.path.basename(file)}:", file=sys.stderr)
        for k in f.keys():
            print(hex(int(k)))


def display(file, counters):
    normalized = True
    with h5py.File(file,'r') as f:
        try:
            fw_ver = f.attrs["firmware_version"]
            dev_type = f.attrs["device_type"]
            print(f"Device: {dev_type}; Firmware: {fw_ver}")
        except KeyError:
            pass
        for c in counters:
            try:
                dset = f[c]
            except KeyError:
                list_datasets(sys.argv[1])
                sys.exit(1)
            name = dset.attrs["name"]
            try:
                normalizer = dset.attrs["normalizer"]
            except KeyError:
                if normalized:
                    print("WARNING: This data is from an older version of ngauge. Plotting un-normalized.", file=sys.stderr)
                    normalized = False
                normalizer = "N/A"
            raw_times = dset[1:, 0]
            time = (raw_times - dset[1, 0]) * 1e-9
            common_current_indexes = np.arange(len(time))

            if normalizer == "N/A":
                normalizer = 1
            elif normalizer == "time":
                normalizer = (np.diff(dset[:, 0] - dset[0, 0])) * 1e-9
            elif normalizer[:3] == "id/":
                normalizer_id = str(int(normalizer.split('/', maxsplit=1)[1], 16))
                normalizer_raw_times = f[normalizer_id][1:, 0]
                _, common_current_indexes, common_normalizer_indexes = \
                    np.intersect1d(raw_times, normalizer_raw_times,
                                   assume_unique=True, return_indices=True)
                time = time[common_current_indexes]
                normalizer = f[normalizer_id][1:, 2][common_normalizer_indexes]
                normalizer = normalizer.astype(np.float64)
                normalizer[normalizer==0] = np.nan

            data = dset[1:, 2][common_current_indexes] / normalizer

            plt.scatter(time, data)
            plt.title(name)
            plt.xlabel("Approx. time (s)")
            plt.plotsize(100, 30)
            plt.show()
            plt.clf()
            print()


if __name__ == '__main__':
    if len(sys.argv) < 3:
        fail()

    if not os.path.isfile(sys.argv[1]):
        fail()

    counters = []
    for cnt in sys.argv[2:]:
        if cnt.isdigit():
            counters.append(cnt)
        elif cnt.startswith("0x") and all(c in string.hexdigits for c in cnt[2:]):
            counters.append(str(int(cnt, 16)))
        else:
            fail()

    display(sys.argv[1], counters)
